/* Procfs interface for the ISA series device driver.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

  v 0.0.0 18 Nov 2002 by Reed Lai
    Gives support to kernel 2.4.
    Steal form IxPIO project.
    Create. */

#include <linux/proc_fs.h>
#include <linux/init.h>

#include "ixisa.h"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
int ixisa_get_info(char *buf, char **start, off_t offset, int buf_size)
#else
int ixisa_get_info(char *buf, char **start, off_t offset, int buf_size,
				   int unused)
#endif
{
	/* read file /proc/ixisa
	 *
	 * Arguments: as /proc filesystem, read <linux/proc_fs.h>
	 *
	 * Returned:  number of written bytes */

	char *p, *q, *l;
	unsigned int i, n;
	ixisa_devinfo_t *r;
	char my_buf[80];

	if (offset > 0)
		return 0;
	/* here, we assume the buf is always large
	   enough to hold all of our info data at
	   one fell swoop */

	p = buf;
	n = buf_size;

	/* export major number */
	sprintf(my_buf, "maj: %d\n", ixisa_major);
	q = my_buf;
	for (; n > 0 && *q != '\0'; --n) {	/* export characters */
		*p++ = *q++;
	}

	/* export module names */
	i = 0;
	l = "mod:";
	for (; n > 0 && *l != '\0'; --n) {
		*p++ = *l++;
	}
	while (ixisa_card[i].id) {	/* scan card list */
		if (ixisa_card[i].present) {	/* find present card */
			if (n > 0) {
				*p++ = ' ';
				--n;
			}
			q = ixisa_card[i].module;
			for (; n > 0 && *q != '\0'; --n) {	/* export card's module name */
				*p++ = *q++;
			}
		}
		++i;
	}
	if (n > 0) {				/* separator */
		*p++ = '\n';
		--n;
	}

	/* export device characters */
	r = ixisa_dev;
	while (r) {
		l = "dev: ";
		for (; n > 0 && *l != '\0'; --n) {	/* export card's module name */
			*p++ = *l++;
		}
		sprintf(my_buf,
				"ixisa%d 0x%lx 0x%x %s\n",
				r->no, r->base, r->id, (char *) _isa_cardname(r->id));
		q = my_buf;
		for (; n > 0 && *q != '\0'; --n) {	/* export characters */
			*p++ = *q++;
		}
		r = r->next;
	}

	return (p - buf - offset);	/* bye bye */
}

static int ixisa_proc_output(char *buf)
{
        char *p;
        unsigned int i = 0;
        ixisa_devinfo_t *r;

        p = buf;

        /* export major number */

        p += sprintf(p, "maj: %d\n", ixisa_major);
        p += sprintf(p, "%s", "mod:");

        while (ixisa_card[i].id) {      /* scan card list */
                if (ixisa_card[i].present) {    /* find present card */
                        p += sprintf(p, " ");
                        p += sprintf(p, "%s", ixisa_card[i].module);
                }
                ++i;
        }
        p += sprintf(p, "\n");


	 /* export device characters */
        r = ixisa_dev;
        while (r) {
                p += sprintf(p, "%s", "dev: ");
                p += sprintf(p, "ixisa%d 0x%lx 0x%x %s\n",
                             r->no, r->base, r->id, (char *) _isa_cardname(r->id));
                r = r->next;
        }

        return  p - buf;
}

static int ixisa_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
        int len = ixisa_proc_output (page);
        if (len <= off+count) *eof = 1;
        *start = page + off;
        len -= off;
        if (len>count)
                len = count;
        if (len<0)
                len = 0;

        return len;
}


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
static struct proc_dir_entry *ixisa_proc_dir;
#else
struct proc_dir_entry ixisa_procfile = {
	low_ino:0,					/* inode number, 0, auto-filled */
	namelen:DEVICE_NAME_LEN,	/* length of proc file name */
	name:DEVICE_NAME,			/* proc file name */
	mode:S_IFREG | S_IRUGO,
	nlink:1,					/* number of links (where the file is
								   referenced) */
	uid:0,						/* user id, 0, root */
	gid:0,						/* group id, 0, root */
	size:80,					/* the size of the file reported by
								   command "ls" */
	get_info:ixisa_get_info,	/* the read function for this file the
								   function called when somebody tries to
								   read something from it */
};
#endif							/* LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) */

void ixisa_proc_exit(void)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	remove_proc_entry(DEVICE_NAME, ixisa_proc_dir);
        remove_proc_entry(ORGANIZATION, NULL);
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
	proc_unregister(&proc_root, ixisa_procfile.low_ino);
#endif
}

int ixisa_proc_init(void)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
        ixisa_proc_dir = proc_mkdir(ORGANIZATION, 0);

        create_proc_read_entry(DEVICE_NAME, 0, ixisa_proc_dir, ixisa_read_proc, NULL);
        return 0;
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
        return proc_register(&proc_root, &ixisa_procfile);
#else
        return proc_register_dynamic(&proc_root, &ixisa_procfile);
#endif
}
