/* Read ADC result by pacer trigger and interrupt.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <signal.h>

#include "ixisa.h"

int fd;

void sig_handler(int sig)
{
	static int counter;
	ixisa_reg_t reg;
	float ai;

	/* analog input */
	reg.id = IXISA_AI;
	reg.mode = IXISA_RM_NORMAL;

	if (++counter == 50) {  /* 1 Hz pacer */
		if (ioctl(fd, IXISA_READ_REG, &reg))
			printf("Failure of analog input.");
		ai = (10 *( (float)reg.value / 0xfff) -5);
		printf("%x %f   ", reg.value, ai);
		counter = 0;
	}
}

int main()
{
	char *dev_file;
	ixisa_reg_t reg;
	ixisa_signal_t sig;
	static struct sigaction act, act_old;
	ixisa_count_t count;

	dev_file = "/dev/ixisa1";

	/* open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of open device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* signal action */
	act.sa_handler = sig_handler;
	sigemptyset(&act.sa_mask);
	sigaddset(&act.sa_mask, SIGALRM);
	if (sigaction(SIGALRM, &act, &act_old)) {
		close(fd);
		puts("Failure of siganl action.");
		return FAILURE;
	}

	/* signal condiction */
	sig.sid = SIGALRM;
	sig.pid = getpid();
	if (ioctl(fd, IXISA_SET_SIG, &sig)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of signal condiction.");
		return FAILURE;
	}

	/* gain for +/- 5V */
	reg.id = IXISA_ADGCR;
	reg.value = 0;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of gain configuration.");
		return FAILURE;
	}

	/* channel 0 */
	reg.id = IXISA_ADMXCR;
	//reg.value = 0;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of channel.");
		return FAILURE;
	}
	usleep(23);
		/* delay 23us for channel switching */

	/* clear board irq status */
	if (ioctl(fd, IXISA_CLEAR_BOARD_IRQ)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of clear irq status.");
		return FAILURE;
	}

	/************************
	 * setup counter 1 & 2  *
	 *                      *
	 * 2MHz / 40000 = 50Hz  *
	 ************************/
	count.id = 1;
	count.value = 200;
	count.mode = _8254CW_MODE_2;
	if (ioctl(fd, IXISA_COUNT_CONF, &count)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of counter 2");
		return FAILURE;
	}

	count.id = 2;
	count.value = 200;
	count.mode = _8254CW_MODE_2;
	if (ioctl(fd, IXISA_COUNT_CONF, &count)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of counter 2");
		return FAILURE;
	}

	/* pacer trigger, IRQ 7, software/interrupt transfer*/
	reg.id = IXISA_ADMCR;
	//reg.value = 0x76;
	reg.value = 0x6;
	if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of configuring interrupt.");
		return FAILURE;
	}

	/* wait for exit */
	puts("Press <enter> to get out.");
	while (getchar() != '\n');

	close(fd);
	sigaction(SIGALRM, &act_old, NULL);
	puts("\nEnd of program.");
	return SUCCESS;
}
