/* Read/write register by ioctl command IXISA_READ_REG and IXISA_WRITE_REG for
   digital I/O.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   v 0.0.0 11 Apr 2003 by Reed Lai
     I do this for Aiur! */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include "ixisa.h"
#include "_isoad32.h"

int main()
{
	int fd;
	char *dev_file;
	ixisa_reg_t sr, fifo;
	int i = 0, x=0;
	int out_1, out_2, out_3, out_4, out_5;
	int in_1, in_2;
	float f;
	
	sr.id = IXISA_SR;
	fifo.id = IXISA_FIFO;
	
	dev_file = "/dev/ixisa1";

	/* open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of open device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* CheckBoard */
	fifo.value = 0;		//clear fifo
	ioctl(fd, IXISA_WRITE_REG, &fifo);
	ioctl(fd, IXISA_WRITE_REG, &fifo);
	ioctl(fd, IXISA_WRITE_REG, &fifo);
	/* Select the fixed-channel mode, channel_0, bipolar and +/-5V range */
	out_1 = (5/16);			// Channel/16
	out_2 = ((5%16) + 0x00 + 0x00);	// Channel%16 + mode + config
	out_5 = ((23*5)-1);		// Settling time *5-1
	out_3 = (out_5/256);
	out_4 = (out_5%256);

	/* SetChannelConfigNoScan */
	ioctl(fd, IXISA_READ_REG, &sr);
	if ((sr.value & 0x01) != 0){
		sr.value = CMD_WRITE_MUX_CONFIG;
		ioctl(fd, IXISA_WRITE_REG, &sr);
		ioctl(fd, IXISA_READ_REG, &sr);
		if((sr.value & 0x01) !=0){
			sr.value = out_1;
			ioctl(fd, IXISA_WRITE_REG, &sr);
			ioctl(fd, IXISA_READ_REG, &sr);
			if((sr.value & 0x01) !=0){
				sr.value = out_2;
				ioctl(fd, IXISA_WRITE_REG, &sr);
				ioctl(fd, IXISA_READ_REG, &sr);
				if((sr.value & 0x01) != 0){
					sr.value = out_3;
					ioctl(fd, IXISA_WRITE_REG, &sr);
					ioctl(fd, IXISA_READ_REG, &sr);
					if((sr.value & 0x01) != 0){
						sr.value = out_4;
						ioctl(fd, IXISA_WRITE_REG, &sr);
					}
				}
			}
		}
	}
	i = 0;
	while (i<30){
		ioctl(fd, IXISA_READ_REG, &sr);
		if((sr.value & 0x02) != 0){
			ioctl(fd, IXISA_READ_REG, &fifo);
			out_2 = fifo.value;
			if (out_2 == CMD_WRITE_MUX_CONFIG){
				printf("SetChannelConfig NoError\n");
				break;
			}else{
				printf("HandshakeError\n");
			}
		}
		i++;
	}
	
	/* ReadChannelConfigNoScan */
	i = 0;
	while(i<10){
		ioctl(fd, IXISA_READ_REG, &sr);
		if ((sr.value & 0x01) != 0){
			sr.value = CMD_READ_MUX_CONFIG;
			ioctl(fd, IXISA_WRITE_REG, &sr);
			break;
		}else printf("CMD error");	//break;
		i++;
	}
	
	while(i<10){
		ioctl(fd, IXISA_READ_REG, &sr);
		if ((sr.value & 0x02) != 0){
			ioctl(fd, IXISA_READ_REG, &fifo);
			in_1 = fifo.value;
			ioctl(fd, IXISA_READ_REG, &sr);
			if ((sr.value & 0x02) != 0){
				ioctl(fd, IXISA_READ_REG, &fifo);
				in_2 = fifo.value;
				printf("config=%x, mode=%x, channel=%x\n", in_2&0xb0, in_2&0x40, (in_1%2) * 16 + in_2%16);
				break;
			}
		}
		i++;
	}

	/* ReadAdNoScan */
	i = 0;
	while (i < 10) {
		ioctl(fd, IXISA_READ_REG, &sr);
		if ((sr.value & 0x01) != 0){
			sr.value = CMD_READ_CURRENT_AD;
			ioctl(fd, IXISA_WRITE_REG, &sr);
			break;
		}
		i++;
	}

	i = 0;
	while (i < 100) {
		ioctl(fd, IXISA_READ_REG, &sr);
		if ((sr.value & 0x02) != 0) {
			ioctl(fd, IXISA_READ_REG, &fifo);
			in_1 = fifo.value;
			ioctl(fd, IXISA_READ_REG, &sr);
			if ((sr.value & 0x02) != 0) {
				ioctl(fd, IXISA_READ_REG, &fifo);
				in_2 = fifo.value;
				out_1 = in_2;
				out_1 = out_1*256 + in_1;
				printf("in_1 %x, out_1 %x\n", in_1, out_1);
				break;
			}
		}
		i++;
	}

	f = (float)(out_1 - 2048)/2048*5.0; //Convert from 12 bits binary data to floating point value.
	printf("data=%x, f=%f\n", out_1, f);

	close(fd);
	return SUCCESS;
}
