/* Read/write fifo by ioctl command IXISA_READ_REG and IXISA_WRITE_REG for
   AD test.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* Channel = 0 --> general AD
 * 	   = 1 --> IoadceII AD
 * Gain	   = 0 --> Gain = 1
 * 	     1 --> Gain = 10
 * 	     2 --> Gain = 100
 * 	     3 --> Gain = 1000
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include "ixisa.h"
#include "_isold.h"

int main()
{
	int fd;
	char *dev_file;
	ixisa_reg_t sr, fifo;
	int i = 0, x=0, ok = 0, err = 0;
	int channel; int gain;
	
	sr.id = IXISA_SR;
	fifo.id = IXISA_FIFO;
	
	dev_file = "/dev/ixisa1";

	/* open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of open device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* Check Board */
	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x04) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	 if ((sr.value & 0x04) != 0) {
		sr.value = CMD_WRITE_DI_MODE;
		ioctl(fd, IXISA_WRITE_REG, &sr);
		sr.value = 0x55;
		ioctl(fd, IXISA_WRITE_REG, &sr);

		ioctl(fd, IXISA_READ_REG, &sr);
		if ((sr.value & 0x04 != 0))
		{
			sr.value = CMD_READ_DI_MODE;
			ioctl(fd, IXISA_WRITE_REG, &sr);
			
			ioctl(fd, IXISA_READ_REG, &sr);
			while ((sr.value & 0x02) ==0)
			{
				ioctl(fd, IXISA_READ_REG, &sr);
			}
		
			ioctl(fd, IXISA_READ_REG, &fifo);
			if (fifo.value != 0x55)
				printf("CheckError, not 0x55, value = %x\n", fifo.value);		
		}else{
			printf("Read  error");
			return FAILURE;
		}
	 }else {
		printf("CheckBoard error\n");
		return FAILURE;
	 }
	
	/* test AD channel 0 : normal AD */ 
	for (i = 0; i < 4; i++) {
		ioctl(fd, IXISA_READ_REG, &sr);
		while ((sr.value & 0x04) == 0) {
			ioctl(fd, IXISA_READ_REG, &sr);
		}
		
		sr.value = CMD_WRITE_PGA;
		ioctl(fd, IXISA_WRITE_REG, &sr);
		sr.value = (0*8 + (i/2)*16 + (i%2)*32);	/* Channel*8 + (Gain/2)*16 + (Gain%2) * 32 */
		ioctl(fd, IXISA_WRITE_REG, &sr);

		ioctl(fd, IXISA_READ_REG, &sr);
		while ((sr.value & 0x04) ==0) {
			ioctl(fd, IXISA_READ_REG, &sr);
		}
		
		sr.value = CMD_READ_PGA;
		ioctl(fd, IXISA_WRITE_REG, &sr);
		
		ioctl(fd, IXISA_READ_REG, &sr);
		while ((sr.value & 0x02) ==0)
		{
			ioctl(fd, IXISA_READ_REG, &sr);
		}
		
		ioctl(fd, IXISA_READ_REG, &fifo);
		i = fifo.value;
		if (i & 0x08){
			channel = 1;
		}else{
			channel = 0;
		}

		i &= 0x30;
		switch (i) {
			case 0x30: 
				gain = 3;  
				break;
			case 0x20:
				gain = 1;
				break;
			case 0x10:
				gain = 2;
				break;
			case 0x00:
				gain = 0;
				break;
		}
		printf("Channel = %d, Gain = %d\n", channel, gain);
	}

	/* test AD channel 1 : normal AD */ 
	for (i = 0; i < 4; i++) {
		ioctl(fd, IXISA_READ_REG, &sr);
		while ((sr.value & 0x04) == 0) {
			ioctl(fd, IXISA_READ_REG, &sr);
		}
		
		sr.value = CMD_WRITE_PGA;
		ioctl(fd, IXISA_WRITE_REG, &sr);
		sr.value = (1*8 + (i/2)*16 + (i%2)*32);	/* Channel*8 + (Gain/2)*16 + (Gain%2) * 32 */
		ioctl(fd, IXISA_WRITE_REG, &sr);

		ioctl(fd, IXISA_READ_REG, &sr);
		while ((sr.value & 0x04) ==0) {
			ioctl(fd, IXISA_READ_REG, &sr);
		}
		
		sr.value = CMD_READ_PGA;
		ioctl(fd, IXISA_WRITE_REG, &sr);
		
		ioctl(fd, IXISA_READ_REG, &sr);
		while ((sr.value & 0x02) ==0)
		{
			ioctl(fd, IXISA_READ_REG, &sr);
		}
		
		ioctl(fd, IXISA_READ_REG, &fifo);
		i = fifo.value;
		if (i & 0x08){
			channel = 1;
		}else{
			channel = 0;
		}

		i &= 0x30;
		switch (i) {
			case 0x30: 
				gain = 3;  
				break;
			case 0x20:
				gain = 1;
				break;
			case 0x10:
				gain = 2;
				break;
			case 0x00:
				gain = 0;
				break;
		}
		printf("Channel = %d, Gain = %d\n", channel, gain);
	}

	/* Set Pacer Timer 20k */
	/* Sampling rate = 1,000,000/Timer+1) Hz
	 *   500  Hz : 2000 uS = 1999
	 *  1.00 KHz : 1000 uS =  999
	 *  2.00 KHz :  500 uS =  499
	 * 10.00 KHz :  100 uS =   99
	 * 20.00 KHz :   50 uS =   49
	 */
	
	ioctl(fd, IXISA_READ_REG, &sr);
	while ((sr.value & 0x04) == 0) {
		ioctl(fd, IXISA_READ_REG, &sr);
	}

	sr.value = CMD_SET_TIMER;
	ioctl(fd, IXISA_WRITE_REG, &sr);
	
	sr.value = ((99 / 256) & 0xff);
	ioctl(fd, IXISA_WRITE_REG, &sr);

	sr.value = ((99 % 256) & 0xff);
	ioctl(fd, IXISA_WRITE_REG, &sr);

	/* Read Pacer Timer */

	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x04) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	
	 sr.value = CMD_READ_TIMER;
	 ioctl(fd, IXISA_WRITE_REG, &sr);
	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x02) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	 ioctl(fd, IXISA_READ_REG, &fifo);
	 i = fifo.value;
	 i *= 256;
	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x02) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	 ioctl(fd, IXISA_READ_REG, &fifo);
	 i += fifo.value;
	 printf("20K : Count = %d\n", i);

	/* Set Pacer Timer 1k */
	ioctl(fd, IXISA_READ_REG, &sr);
	while ((sr.value & 0x04) == 0) {
		ioctl(fd, IXISA_READ_REG, &sr);
	}

	sr.value = CMD_SET_TIMER;
	ioctl(fd, IXISA_WRITE_REG, &sr);
	
	sr.value = ((999 / 256) & 0xff);
	ioctl(fd, IXISA_WRITE_REG, &sr);

	sr.value = ((999 % 256) & 0xff);
	ioctl(fd, IXISA_WRITE_REG, &sr);

	/* Read Pacer Timer */

	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x04) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	
	 sr.value = CMD_READ_TIMER;
	 ioctl(fd, IXISA_WRITE_REG, &sr);
	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x02) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	 ioctl(fd, IXISA_READ_REG, &fifo);
	 i = fifo.value;
	 i *= 256;
	 ioctl(fd, IXISA_READ_REG, &sr);
	 while ((sr.value & 0x02) == 0) {
		 ioctl(fd, IXISA_READ_REG, &sr);
	 }
	 ioctl(fd, IXISA_READ_REG, &fifo);
	 i += fifo.value;
	 printf("1K : Count = %d\n", i);

	close(fd);
	return SUCCESS;
}

