/* Example of interrupt handle for ISO-730.

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   JP2 Interrupt Trigger
     +-------+
     | +-+   |
     | |o| o |
     | | |   |
     | |o| o |
     | +-+   |
     +-------+
   Rising edge (Default)

   JP3 Interrupt Source
     +-----------------+
     |          +-+    |  
     |  o    o  |o|  o |
     |  o    o  |o|  o |
     |          +-+    |
     +-----------------+
      IDI0 IDI1 DI0 DI1    

      Select DI0 Connector CN4 Pin1
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <signal.h>

#include "ixisa.h"

#define MYSIG 33

unsigned sig_counter;

void sig_handler(int sig)
{
	++sig_counter;
	putchar('+');
}

int main()
{
	int fd, i = 0;
	char *dev_file;
	ixisa_reg_t reg;
	ixisa_signal_t sig;
	int x;

	static struct sigaction act, act_old;

	sig_counter = 0;
	dev_file = "/dev/ixisa1";

	/* open device file */
	fd = open(dev_file, O_RDWR);
	if (fd < 0) {
		printf("Failure of openning device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* signal action */
	act.sa_handler = sig_handler;
	sigemptyset(&act.sa_mask);
	sigaddset(&act.sa_mask, MYSIG);
	if (sigaction(MYSIG, &act, &act_old)) {
		close(fd);
		puts("Failure of signal action.");
		return FAILURE;
	}

	/* signal condiction */
	sig.sid = MYSIG;
	sig.pid = getpid();
	if (ioctl(fd, IXISA_SET_SIG, &sig)) {
		close(fd);
		sigaction(MYSIG, &act_old, NULL);
		puts("Failure of signal condition.");
		return FAILURE;
	}

	/* rocking time */
	reg.id = IXISA_DIO_A;
	puts("please wait a moment...\n");
	while (i++ < 100) {
		reg.value = x++;
		if (ioctl(fd, IXISA_WRITE_REG, &reg)) {
			close(fd);
			sigaction(MYSIG, &act_old, NULL);
			puts("Failure of rising.");
			return FAILURE;
		}
		usleep(50000);
		ioctl(fd, IXISA_READ_REG, &reg);
		printf("%x ", reg.value);
	}
	printf("\nGet interrupt signal for %d times.\nPush a key to get out: ", sig_counter);
	getchar();

	close(fd);
	sigaction(MYSIG, &act_old, NULL);
	puts("\nEnd of program.");
	return SUCCESS;
}
