/* Example of Digital I/O for TMC-10

   Author: Reed Lai

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* test the input  port 						 */
/* IO BASE address = 0x200, all DIP switchs of SW1 in OFF position	 */
/* CN1.15 connect to CN1.2  & CN1.8   --> DO.0 TO DI1.0 & DI2.0 	 */
/* CN1.34 connect to CN1.4  & CN1.10  --> DO.1 TO DI1.1 & DI2.1 	 */
/* CN1.16 connect to CN1.5  & CN1.11  --> DO.2 TO DI1.2 & DI2.2 	 */
/* CN1.35 connect to CN1.7  & CN1.13  --> DO.3 TO DI1.3 & DI2.3 	 */
/* CN1.17 connect to CN1.21 & CN1.27  --> DO.4 TO DI1.4 & DI2.4 	 */
/* CN1.36 connect to CN1.22 & CN1.28  --> DO.5 TO DI1.5 & DI2.5 	 */
/* CN1.18 connect to CN1.24 & CN1.30  --> DO.6 TO DI1.6 & DI2.6 	 */
/* CN1.37 connect to CN1.25 & CN1.31  --> DO.7 TO DI1.7 & DI2.7 	 */

/* *INDENT-OFF* */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <string.h>

#include "ixisa.h"

int main()
{
	int fd_dev;
	ixisa_reg_t rdi, rdi1, rdo;
	char *dev_file;
	
	dev_file = "/dev/ixisa1";
		/* change this for your device entry */

	/* open device */
	fd_dev = open(dev_file, O_RDWR);
	if (fd_dev < 0) {
		printf("Cannot open device file \"%s.\"\n", dev_file);
		return FAILURE;
	}

	/* DIO */
	rdi.id = IXISA_DI_A;
	rdi1.id = IXISA_DI_B;
	rdo.id = IXISA_DO_A;
	rdo.value = 1;

	while (getchar() != 27) {
		if (ioctl(fd_dev, IXISA_WRITE_REG, &rdo)) {  /* DO */
			puts("Failure of digital output.");
			close(fd_dev);
			return FAILURE;
		}
		printf("DO = 0x%2x\t", rdo.value);
		if (ioctl(fd_dev, IXISA_READ_REG, &rdi)) {   /* DI 0-7 */
			puts("Failure of digital output.");
			close(fd_dev);
			return FAILURE;
		}
		printf("DI = 0x%2x\t", rdi.value);
		if (ioctl(fd_dev, IXISA_READ_REG, &rdi1)) {  /* DI 8-15 */
			puts("Failure of digital input.");
			return FAILURE;
		}
		printf("DI1 = 0x%2x\tESC-enter to get out, enter to continue\n", rdi1.value);
		rdo.value == 0xff ? rdo.value = 1 : (rdo.value += 1);
	}

	close(fd_dev);
	return SUCCESS;
}
/* *INDENT-ON* */
